@@ -23,6 +23,8 @@ mod test_readme {
     external_doc_test!(include_str!("../README.md"));
 }
 
+use std::fmt;
+
 pub use tagger::upgrade_write;
 
 pub use crop::Crop;
@@ -30,48 +32,56 @@ pub use crop::Croppable;
 mod crop;
 
 mod render;
-mod util;
-use std::fmt;
+
+pub mod util;
 
 ///The width of the svg tag.
 const WIDTH: f64 = 800.0;
 ///The height of the svg tag.
 const HEIGHT: f64 = 500.0;
 
-trait PlotTrait {
+trait PlotTrait<X: PlotNum, Y: PlotNum> {
     fn write_name(&self, a: &mut dyn fmt::Write) -> fmt::Result;
 
-    fn iter_first(&mut self) -> &mut dyn Iterator<Item = [f64; 2]>;
-    fn iter_second(&mut self) -> &mut dyn Iterator<Item = [f64; 2]>;
+    fn iter_first(&mut self) -> &mut dyn Iterator<Item = (X, Y)>;
+    fn iter_second(&mut self) -> &mut dyn Iterator<Item = (X, Y)>;
 }
 
+use std::marker::PhantomData;
+
 use fmt::Display;
-struct PlotStruct<I: Iterator<Item = [f64; 2]> + Clone, F: Display> {
+struct PlotStruct<X: PlotNum, Y: PlotNum, I: Iterator<Item = (X, Y)> + Clone, F: Display> {
     first: I,
     second: I,
     func: F,
+    _p: PhantomData<(X, Y)>,
 }
 
-impl<I: Iterator<Item = [f64; 2]> + Clone, F: Display> PlotStruct<I, F> {
+impl<X: PlotNum, Y: PlotNum, I: Iterator<Item = (X, Y)> + Clone, F: Display>
+    PlotStruct<X, Y, I, F>
+{
     fn new(it: I, func: F) -> Self {
         let it2 = it.clone();
         PlotStruct {
             first: it,
             second: it2,
             func,
+            _p: PhantomData,
         }
     }
 }
 
-impl<D: Iterator<Item = [f64; 2]> + Clone, F: Display> PlotTrait for PlotStruct<D, F> {
+impl<X: PlotNum, Y: PlotNum, D: Iterator<Item = (X, Y)> + Clone, F: Display> PlotTrait<X, Y>
+    for PlotStruct<X, Y, D, F>
+{
     fn write_name(&self, a: &mut dyn fmt::Write) -> fmt::Result {
         write!(a, "{}", self.func)
     }
-    fn iter_first(&mut self) -> &mut dyn Iterator<Item = [f64; 2]> {
+    fn iter_first(&mut self) -> &mut dyn Iterator<Item = (X, Y)> {
         &mut self.first
     }
 
-    fn iter_second(&mut self) -> &mut dyn Iterator<Item = [f64; 2]> {
+    fn iter_second(&mut self) -> &mut dyn Iterator<Item = (X, Y)> {
         &mut self.second
     }
 }
@@ -83,9 +93,9 @@ enum PlotType {
     LineFill,
 }
 
-struct Plot<'a> {
+struct Plot<'a, X: PlotNum, Y: PlotNum> {
     plot_type: PlotType,
-    plots: Box<dyn PlotTrait + 'a>,
+    plots: Box<dyn PlotTrait<X, Y> + 'a>,
 }
 
 ///
@@ -174,86 +184,76 @@ pub const DIMENSIONS: [usize; 2] = [800, 500];
 
 /// Iterators that are passed to the [`Plotter`] plot functions must produce
 /// items that implement this trait.
-pub trait Plottable {
+pub trait Plottable<X: PlotNum, Y: PlotNum> {
     /// Produce one plot
-    fn make_plot(self) -> [f64; 2];
+    fn make_plot(self) -> (X, Y);
 }
 
-/// Convert other primitive types to f64 using this trait.
-/// Precision loss is considered acceptable, since this is just for visual human eyes.
-pub trait AsF64: Copy {
-    fn as_f64(&self) -> f64;
-}
-
-impl<T: AsF64> AsF64 for &T {
-    fn as_f64(&self) -> f64 {
-        AsF64::as_f64(*self)
-    }
-}
-macro_rules! impl_into_plotnum {
-    ($U: ty ) => {
-        impl AsF64 for $U {
-            fn as_f64(&self) -> f64 {
-                *self as f64
-            }
-        }
-    };
-}
-
-impl_into_plotnum!(f32);
-impl_into_plotnum!(f64);
-impl_into_plotnum!(i8);
-impl_into_plotnum!(u8);
-impl_into_plotnum!(i16);
-impl_into_plotnum!(u16);
-impl_into_plotnum!(i32);
-impl_into_plotnum!(u32);
-impl_into_plotnum!(i64);
-impl_into_plotnum!(u64);
-impl_into_plotnum!(i128);
-impl_into_plotnum!(u128);
-impl_into_plotnum!(isize);
-impl_into_plotnum!(usize);
-
-impl<T: AsF64> Plottable for [T; 2] {
-    fn make_plot(self) -> [f64; 2] {
+impl<T: PlotNum> Plottable<T, T> for [T; 2] {
+    fn make_plot(self) -> (T, T) {
         let [x, y] = self;
-        [x.as_f64(), y.as_f64()]
+        (x, y)
     }
 }
 
-impl<T: AsF64> Plottable for &[T; 2] {
-    fn make_plot(self) -> [f64; 2] {
-        let [x, y] = self;
-        [x.as_f64(), y.as_f64()]
+impl<T: PlotNum> Plottable<T, T> for &[T; 2] {
+    fn make_plot(self) -> (T, T) {
+        let [x, y] = *self;
+        (x, y)
     }
 }
 
-impl<A: AsF64, B: AsF64> Plottable for (A, B) {
-    fn make_plot(self) -> [f64; 2] {
-        let (x, y) = self;
-        [x.as_f64(), y.as_f64()]
+impl<A: PlotNum, B: PlotNum> Plottable<A, B> for (A, B) {
+    fn make_plot(self) -> (A, B) {
+        self
     }
 }
 
-impl<A: AsF64, B: AsF64> Plottable for &(A, B) {
-    fn make_plot(self) -> [f64; 2] {
-        let (x, y) = self;
-        [x.as_f64(), y.as_f64()]
+impl<A: PlotNum, B: PlotNum> Plottable<A, B> for &(A, B) {
+    fn make_plot(self) -> (A, B) {
+        *self
     }
 }
 
 ///
 /// Create a Plotter
 ///
-pub fn plot<'a>(
+pub fn plot<'a, X: PlotNum, Y: PlotNum>(
     title: impl Display + 'a,
     xname: impl Display + 'a,
     yname: impl Display + 'a,
-) -> Plotter<'a> {
+) -> Plotter<'a, X, Y> {
     Plotter::new(title, xname, yname)
 }
 
+trait MyFmt<X: PlotNum> {
+    fn write(
+        &self,
+        formatter: &mut std::fmt::Formatter,
+        value: X,
+        step: Option<X>,
+    ) -> std::fmt::Result;
+}
+
+struct Foo<A, X>(A, PhantomData<X>);
+impl<X: PlotNum, A: Fn(&mut std::fmt::Formatter, X, Option<X>) -> std::fmt::Result> Foo<A, X> {
+    fn new(a: A) -> Foo<A, X> {
+        Foo(a, PhantomData)
+    }
+}
+impl<X: PlotNum, A: Fn(&mut std::fmt::Formatter, X, Option<X>) -> std::fmt::Result> MyFmt<X>
+    for Foo<A, X>
+{
+    fn write(
+        &self,
+        formatter: &mut std::fmt::Formatter,
+        value: X,
+        step: Option<X>,
+    ) -> std::fmt::Result {
+        (self.0)(formatter, value, step)
+    }
+}
+
 /// Keeps track of plots.
 /// User supplies iterators that will be iterated on when
 /// render is called.
@@ -263,29 +263,32 @@ pub fn plot<'a>(
 /// * The axis line SVG elements belong to the `poloto_axis_lines` class.
 /// * The background belongs to the `poloto_background` class.
 ///
-pub struct Plotter<'a> {
+pub struct Plotter<'a, X: PlotNum + 'a, Y: PlotNum + 'a> {
     title: Box<dyn fmt::Display + 'a>,
     xname: Box<dyn fmt::Display + 'a>,
     yname: Box<dyn fmt::Display + 'a>,
-    plots: Vec<Plot<'a>>,
-    xmarkers: Vec<f64>,
-    ymarkers: Vec<f64>,
+    plots: Vec<Plot<'a, X, Y>>,
+    xmarkers: Vec<X>,
+    ymarkers: Vec<Y>,
     num_css_classes: Option<usize>,
     preserve_aspect: bool,
+    xtick_fmt: Box<dyn MyFmt<X> + 'a>,
+    ytick_fmt: Box<dyn MyFmt<Y> + 'a>,
 }
 
-impl<'a> Plotter<'a> {
+impl<'a, X: PlotNum, Y: PlotNum> Plotter<'a, X, Y> {
     ///
     /// Create a plotter with the specified element.
     ///
     /// ```
-    /// let p = poloto::Plotter::new("title", "x", "y");
+    /// let mut p = poloto::Plotter::new("title", "x", "y");
+    /// p.line("",[[1,1]]);
     /// ```
     pub fn new(
         title: impl Display + 'a,
         xname: impl Display + 'a,
         yname: impl Display + 'a,
-    ) -> Plotter<'a> {
+    ) -> Plotter<'a, X, Y> {
         Plotter {
             title: Box::new(title),
             xname: Box::new(xname),
@@ -295,6 +298,8 @@ impl<'a> Plotter<'a> {
             ymarkers: Vec::new(),
             num_css_classes: Some(8),
             preserve_aspect: false,
+            xtick_fmt: Box::new(Foo::<_, X>::new(move |a, b, c| b.fmt_tick(a, c))),
+            ytick_fmt: Box::new(Foo::<_, Y>::new(move |a, b, c| b.fmt_tick(a, c))),
         }
     }
     /// Create a line from plots using a SVG polyline element.
@@ -309,7 +314,7 @@ impl<'a> Plotter<'a> {
     where
         I: IntoIterator,
         I::IntoIter: Clone + 'a,
-        I::Item: Plottable,
+        I::Item: Plottable<X, Y>,
     {
         self.plots.push(Plot {
             plot_type: PlotType::Line,
@@ -333,7 +338,7 @@ impl<'a> Plotter<'a> {
     where
         I: IntoIterator,
         I::IntoIter: Clone + 'a,
-        I::Item: Plottable,
+        I::Item: Plottable<X, Y>,
     {
         self.plots.push(Plot {
             plot_type: PlotType::LineFill,
@@ -359,7 +364,7 @@ impl<'a> Plotter<'a> {
     where
         I: IntoIterator,
         I::IntoIter: Clone + 'a,
-        I::Item: Plottable,
+        I::Item: Plottable<X, Y>,
     {
         self.plots.push(Plot {
             plot_type: PlotType::Scatter,
@@ -384,7 +389,7 @@ impl<'a> Plotter<'a> {
     where
         I: IntoIterator,
         I::IntoIter: Clone + 'a,
-        I::Item: Plottable,
+        I::Item: Plottable<X, Y>,
     {
         self.plots.push(Plot {
             plot_type: PlotType::Histo,
@@ -404,10 +409,10 @@ impl<'a> Plotter<'a> {
     /// plotter.line("", &data);
     ///
     /// // Include origin in the graph.
-    /// plotter.xmarker(0).ymarker(0);
+    /// plotter.xmarker(0.0).ymarker(0.0);
     /// ```
-    pub fn xmarker<A: AsF64>(&mut self, marker: A) -> &mut Self {
-        self.xmarkers.push(marker.as_f64());
+    pub fn xmarker(&mut self, marker: X) -> &mut Self {
+        self.xmarkers.push(marker);
         self
     }
 
@@ -419,10 +424,10 @@ impl<'a> Plotter<'a> {
     /// plotter.line("", &data);
     ///
     /// // Include origin in the graph.
-    /// plotter.xmarker(0).ymarker(0);
+    /// plotter.xmarker(0.0).ymarker(0.0);
     /// ```
-    pub fn ymarker<A: AsF64>(&mut self, marker: A) -> &mut Self {
-        self.ymarkers.push(marker.as_f64());
+    pub fn ymarker(&mut self, marker: Y) -> &mut Self {
+        self.ymarkers.push(marker);
         self
     }
 
@@ -456,12 +461,46 @@ impl<'a> Plotter<'a> {
     /// Move a plotter out from behind a mutable reference leaving
     /// an empty plotter.
     ///
-    pub fn move_into(&mut self) -> Plotter<'a> {
+    pub fn move_into(&mut self) -> Plotter<'a, X, Y> {
         let mut empty = crate::Plotter::new("", "", "");
         core::mem::swap(&mut empty, self);
         empty
     }
 
+    ///
+    /// Overrides the [`PlotNum::fmt_tick`] function.
+    /// The callback function provided will get called on each
+    /// interval tick to be drawn. The callback function is passed
+    /// the value of the interval, as well as the step-size.
+    /// This function is also called to display `k` and `j` values
+    /// for when the magnitudes of of the plots are extreme.
+    /// In those cases, the step-size is not provided and `None` is passed.
+    ///
+    pub fn xinterval_fmt(
+        &mut self,
+        a: impl Fn(&mut std::fmt::Formatter, X, Option<X>) -> std::fmt::Result + 'a,
+    ) -> &mut Self {
+        self.xtick_fmt = Box::new(Foo::new(a));
+        self
+    }
+
+    ///
+    /// Overrides the [`PlotNum::fmt_tick`] function.
+    /// The callback function provided will get called on each
+    /// interval tick to be drawn. The callback function is passed
+    /// the value of the interval, as well as the step-size.
+    /// This function is also called to display `k` and `j` values
+    /// for when the magnitudes of of the plots are extreme.
+    /// In those cases, the step-size is not provided and `None` is passed.
+    ///
+    pub fn yinterval_fmt(
+        &mut self,
+        a: impl Fn(&mut std::fmt::Formatter, Y, Option<Y>) -> std::fmt::Result + 'a,
+    ) -> &mut Self {
+        self.ytick_fmt = Box::new(Foo::new(a));
+        self
+    }
+
     ///
     /// Use the plot iterators to write out the graph elements.
     /// Does not add a svg tag, or any styling elements.
@@ -485,7 +524,7 @@ impl<'a> Plotter<'a> {
     }
 
     ///
-    /// Make a graph with a svg tag and a simple dark css theme.
+    /// Make a graph with a svg tag and a simple css theme.
     ///
     /// ```
     /// let data = [[1.0,4.0], [2.0,5.0], [3.0,6.0]];
@@ -534,7 +573,7 @@ impl<'a> Plotter<'a> {
         w.into_writer()
     }
 }
-
+/*
 pub struct Renderer {}
 impl Renderer {
     pub fn render<T: fmt::Write>(&mut self, a: T) -> T {
@@ -546,6 +585,7 @@ impl Renderer {
         w.into_writer()
     }
 }
+*/
 
 /// Shorthand for `moveable_format(move |w|write!(w,...))`
 /// Similar to `format_args!()` except has a more flexible lifetime.
@@ -585,3 +625,103 @@ pub fn minify(a: &str) -> impl fmt::Display + '_ + Send + Sync {
     })
 }
 */
+
+///
+/// A disconnectable number. A number that can me marked as a hole to signify that there is a disconnect in plots.
+/// See [`Croppable`]
+///
+pub trait DiscNum: PlotNum {
+    /// Create a hole value.
+    fn hole() -> Self;
+}
+
+///
+/// A plottable number. In order to be able to plot a number, we need information on how
+/// to display it as well as the interval ticks.
+///
+pub trait PlotNum: PartialOrd + Copy + std::fmt::Display {
+    /// Is this a hole value to inject discontinuty?
+    fn is_hole(&self) -> bool {
+        false
+    }
+
+    ///
+    /// Given an ideal number of intervals across the min and max values,
+    /// Calculate information related to where the interval ticks should go.
+    ///
+    fn compute_ticks(ideal_num_steps: u32, range: [Self; 2]) -> TickInfo<Self>;
+
+    /// If there is only one point in a graph, or no point at all,
+    /// the range to display in the graph.
+    fn unit_range() -> [Self; 2];
+
+    /// Provided a min and max range, scale the current value against max.
+    fn scale(&self, val: [Self; 2], max: f64) -> f64;
+
+    /// Used to display a tick
+    /// Before overriding this, consider using [`crate::Plotter::xinterval_fmt`] and [`crate::Plotter::yinterval_fmt`].
+    fn fmt_tick(
+        &self,
+        formatter: &mut std::fmt::Formatter,
+        _step: Option<Self>,
+    ) -> std::fmt::Result {
+        write!(formatter, "{}", self)
+    }
+
+    /// Compute the exact size of the ticks.
+    /// This can be overridden such that it always returns `None`.
+    /// This way there will be no dashes, and it will just be a solid line.
+    fn dash_size(
+        ideal_dash_size: f64,
+        tick_info: &TickInfo<Self>,
+        range: [Self; 2],
+        max: f64,
+    ) -> Option<f64>;
+}
+
+///
+/// One interval tick
+///
+#[derive(Clone)]
+pub struct Tick<I> {
+    pub position: I,
+    /// If [`TickInfo::display_relative`] is `None`, then this has the same value as [`Tick::position`]
+    pub value: I,
+}
+
+///
+/// Information on the properties of all the interval ticks for one dimension.
+///
+#[derive(Clone)]
+pub struct TickInfo<I> {
+    /// List of the position of each tick to be displayed.
+    pub ticks: Vec<Tick<I>>,
+    /// The difference between two adjacent ticks
+    pub step: I,
+    /// The starting tick position
+    pub start_step: I,
+    /// The number of dashes between two ticks must be a multiple of this number.
+    pub dash_multiple: u32,
+
+    /// If we want to display the tick values relatively, this will
+    /// have the base start to start with.
+    pub display_relative: Option<I>,
+}
+impl<I> TickInfo<I> {
+    pub fn map<J>(self, func: impl Fn(I) -> J) -> TickInfo<J> {
+        TickInfo {
+            ticks: self
+                .ticks
+                .into_iter()
+                .map(|x| Tick {
+                    position: func(x.position),
+                    value: func(x.value),
+                })
+                .collect(),
+            step: func(self.step),
+            start_step: func(self.start_step),
+            dash_multiple: self.dash_multiple,
+            display_relative: self.display_relative.map(|x| func(x)),
+        }
+    }
+}
